==> pitaya-framework-0.0.9.tgz: package/src/app.ts <==
import { EventAggregator }                from "aurelia-event-aggregator";
import { inject }                         from "aurelia-framework";
import { Router, RouterConfiguration }    from "aurelia-router";
import { IAppConfig, IRoutesConfigRoute } from "../interfaces.d";
import { PitayaAppConfiguration }         from "./app-configuration";
import { PitayaAppViewCustomElement }     from "./components/pitaya-app-view";
import { delay }                          from "./utils";
import { PitayaView }                     from "./view";

debugger;

const _ROUTER_CONFIGURED_CALLBACK_NAME: string = "routerConfigured";

const _ONLINE_CALLBACK_NAME: string  = "onOnline";
const _OFFLINE_CALLBACK_NAME: string = "onOffline";

const _APP_VIEW_HTML_ELEMENT_NAME: string = "pitaya-app-view";

const _TRANSITION_TOP_LEVEL: string    = "pitaya-page-transition--top-level";
const _TRANSITION_TOP_TO_CHILD: string = "pitaya-page-transition--top-to-child";
const _TRANSITION_CHILD_TO_TOP: string = "pitaya-page-transition--child-to-top";
const _TRANSITIONS                     = [
	_TRANSITION_TOP_LEVEL,
	_TRANSITION_TOP_TO_CHILD,
	_TRANSITION_CHILD_TO_TOP
];

// const _VIEW_ENTERING_CLASS: string = "au-enter-active";
// const _VIEW_LEAVING_CLASS: string  = "au-leave-active";
const _VIEW_CONTROLS_CLASS: string = "pitaya-app-view--controls-blocked";

const _META_THEME_COLOR_SELECTOR: string = "meta[name=theme-color]";
// const _PRIMARY_COLOR_CSS_VARIABLE_NAME: string = "--mdc-theme-primary";

// const _scrollRestorationList: number[] = [];

let _currentAppInstance: PitayaApp        = null;
let _backButtonOverrideCallback: Function = null;
let _enableScroll: boolean                = true;

export const overrideBrowserBackButton = function( callback: Function )
{
	if( _backButtonOverrideCallback )
	{
		window.removeEventListener( "popstate", _backButtonOverrideCallback as EventListenerOrEventListenerObject );

		_backButtonOverrideCallback = null;
	}

	if( !callback )
	{
		window.history.back();

		return;
	}

	const handler = function()
	{
		window.history.pushState( {}, "" );

		callback();
	};

	_backButtonOverrideCallback = handler;

	window.addEventListener( "popstate", handler );

	window.history.pushState( {}, "" );
};

export const enableScrolling = function( enable: boolean = true )
{
	const enableScroll = typeof enable === "boolean" ? enable : !_enableScroll;
	const { body }     = document;

	_enableScroll = enableScroll;

	if( enableScroll )
	{
		body.style.removeProperty( "overflow" );
		body.style.removeProperty( "height" );

		return;
	}

	body.style.setProperty( "overflow", "hidden" );
	body.style.setProperty( "height", "100%" );
};

export const setStatusBarColor = function( color: string )
{
	_currentAppInstance.setStatusBarColor.call( _currentAppInstance, color );
};

export const navigate = function( routeName: string, options?: any )
{
	return _currentAppInstance.navigate.call( _currentAppInstance, routeName, options );
};

export const enableControls = function( enable: boolean = true )
{
	_currentAppInstance.enableControls = enable;
};

@inject( EventAggregator )
export abstract class PitayaApp
{
	static get instance()
	{
		return _currentAppInstance;
	}

	protected _router: Router;
	protected _routerConfig: RouterConfiguration;
	protected _settings                             = {
		title         : "" as string,
		configuration : {} as string | IAppConfig,
		routes        : {} as string | IRoutesConfigRoute[],
	};
	private _themeColorMetaElement: HTMLMetaElement = document.querySelector( _META_THEME_COLOR_SELECTOR );
	private _viewTransition: string                 = "";

	get viewTransition(): string
	{
		return this._viewTransition;
	}

	set viewTransition( value: string )
	{
		const { rootElement } = this._appView;

		if( !rootElement )
		{
			return;
		}

		this._viewTransition = value;

		const { classList } = rootElement;

		if( classList.contains( value ) )
		{
			return;
		}

		classList.remove( ..._TRANSITIONS );
		classList.add( value );
	}

	private _appView: PitayaAppViewCustomElement;

	get appView(): PitayaAppViewCustomElement
	{
		return this._appView;
	}

	private _enableControls: boolean = false;

	get enableControls(): boolean
	{
		return this._enableControls;
	}

	set enableControls( value: boolean )
	{
		const method = value ? "remove" : "add";

		this._enableControls = value;

		this._appView.rootElement.classList[ method ]( _VIEW_CONTROLS_CLASS );
	}

	get enableScroll(): boolean
	{
		return _enableScroll;
	}

	get currentView(): PitayaView
	{
		return PitayaView.instance || this._router.currentInstruction.viewPortInstructions.default.component.viewModel;
	}

	get isOnline()
	{
		return this._appView.isOnline;
	}

	get theme()
	{
		return this._themeColorMetaElement.getAttribute( "content" );
	}

	protected constructor( private _eventAggregator: EventAggregator )
	{
		_currentAppInstance = this;
	}

	/**
	 * @aurelia hook
	 * Configures global routes and router settings
	 */
	public async configureRouter( config: RouterConfiguration, router: Router ): Promise<void>
	{
		const routs         = await this._getRoutes();
		const [ mainRoute ] = routs;

		/* This adjustment is necessary to prevent aurelia from interfering with our scroll restoration mechanism */
		( router.history as any ).history.scrollRestoration = "manual";

		config.title = this._settings.title;
		config.map( routs );
		config.mapUnknownRoutes( mainRoute.moduleId );

		this._router       = router;
		this._routerConfig = config;

		this._setRouteChangeHandler( config );
		// config.addPostRenderStep( _postRenderStep );

		this[ _ROUTER_CONFIGURED_CALLBACK_NAME ] && this[ _ROUTER_CONFIGURED_CALLBACK_NAME ]();
	}

	/**
	 * @aurelia hook
	 */
	public async activate(): Promise<void>
	{
		await this._initSettings();
	}

	/**
	 * @aurelia hook
	 */
	public created( parentContext, context )
	{
		this._appView = context.controllers.find( controller => controller.behavior.elementName === _APP_VIEW_HTML_ELEMENT_NAME ).viewModel;
	}

	/**
	 * @aurelia hook
	 */
	public bind()
	{
		this._initNetworkChangeHandler();
	}

	/**
	 * @aurelia hook
	 */
	public attached()
	{

	}

	/* TODO: Neu machen! */
	public async navigate( routeName: string, options?: any )
	{
		if( !this._router.hasRoute( routeName ) )
		{
			if( !_isExternalLink( routeName ) )
			{
				throw new Error( `Navigation: Route '${routeName}' not found.` );
			}

			const win = window.open( routeName, "_blank" );

			win.focus();

			return;
		}

		const { route, nav } = this._router.routes.find( ( { name } ) => name === routeName );

		/* There is a bug when changing routes the aurelia way, so I just do it manually */
		location.href = `#/${route}`;
	}

	public navigateBack()
	{
		return this._router.navigateBack();
	}

	public setStatusBarColor( color: string )
	{
		this._themeColorMetaElement.setAttribute( "content", color );
	}

	private _setRouteChangeHandler( config: RouterConfiguration )
	{
		this._eventAggregator.subscribe( "router:navigation:processing", this._handleRouteProcessing.bind( this ) );
		this._eventAggregator.subscribe( "router:navigation:success", this._handleRouteSuccess.bind( this ) );
	}

	private _handleRouteProcessing( { instruction } )
	{
		if( !instruction.previousInstruction )
		{
			return;
		}

		const oldOrder = 1 * ( instruction.previousInstruction.config.navModel.order || 0 );
		const newOrder = 1 * ( instruction.config.navModel.order || 0 );

		if( oldOrder > newOrder )
		{
			this.viewTransition = _TRANSITION_CHILD_TO_TOP;
		}
		else if( oldOrder < newOrder )
		{
			this.viewTransition = _TRANSITION_TOP_TO_CHILD;
		}
		else
		{
			this.viewTransition = _TRANSITION_TOP_LEVEL;
		}
	}

	private _handleRouteSuccess( event: Event )
	{
		this._appView.currentViewName = this._router.currentInstruction.config.name;
	}

	private async _initSettings()
	{
		if( !this._settings.configuration )
		{
			return;
		}

		let settings;

		if( typeof this._settings.configuration === "string" )
		{
			const file = await fetch( this._settings.configuration );
			settings   = await file.json();
		}

		PitayaAppConfiguration.set( settings || this._settings.configuration );
	}

	private async _getRoutes(): Promise<IRoutesConfigRoute[]>
	{
		if( typeof this._settings.routes === "string" )
		{
			const file      = await fetch( this._settings.routes );
			const { routs } = await file.json();

			return routs;
		}

		return this._settings.routes;
	}

	private _initNetworkChangeHandler()
	{
		this._appView.isOnline = navigator.onLine;

		window.addEventListener( "online", this._onOnline.bind( this ) );
		window.addEventListener( "offline", this._onOffline.bind( this ) );
	}

	private async _onOnline()
	{
		this._appView.isOnline = true;

		const { currentView } = this;

		if( _ONLINE_CALLBACK_NAME in currentView )
		{
			return currentView[ _ONLINE_CALLBACK_NAME ]();
		}

		( _ONLINE_CALLBACK_NAME in this ) && this[ _ONLINE_CALLBACK_NAME ]();

		await delay( 100 );

		PitayaView.instance.setTheme();
	}

	private async _onOffline()
	{
		this._appView.isOnline = false;

		const { currentView } = this;

		if( _OFFLINE_CALLBACK_NAME in currentView )
		{
			return currentView[ _OFFLINE_CALLBACK_NAME ]();
		}

		( _OFFLINE_CALLBACK_NAME in this ) && this[ _OFFLINE_CALLBACK_NAME ]();

		await delay( 100 );

		PitayaView.instance.setTheme();
	}
}

/**
 * @see Jon´s answer -> https://stackoverflow.com/questions/6238351/fastest-way-to-detect-external-urls
 * @param link {string}
 */
const _isExternalLink = function( link: string ): boolean
{
	const locationHref = location.href.replace( "http://", "" )
								 .replace( "https://", "" )
								 .split( "/" )[ 0 ];

	const refinedLink = link.replace( "http://", "" ).replace( "https://", "" )
							.split( "/" )[ 0 ];

	return locationHref !== refinedLink;
};

==> pitaya-framework-0.0.14.tgz: package/src/app.ts <==
import { EventAggregator }                from "aurelia-event-aggregator";
import { inject }                         from "aurelia-framework";
import { Router, RouterConfiguration }    from "aurelia-router";
import { IAppConfig, IRoutesConfigRoute } from "../interfaces.d";
import { PitayaAppConfiguration }         from "./app-configuration";
import { PitayaAppViewCustomElement }     from "./components/pitaya-app-view";
import { delay }                          from "./utils";
import { PitayaView }                     from "./view";

const _ROUTER_CONFIGURED_CALLBACK_NAME: string = "routerConfigured";

const _ONLINE_CALLBACK_NAME: string  = "onOnline";
const _OFFLINE_CALLBACK_NAME: string = "onOffline";

const _APP_VIEW_HTML_ELEMENT_NAME: string = "pitaya-app-view";

const _TRANSITION_TOP_LEVEL: string    = "pitaya-page-transition--top-level";
const _TRANSITION_TOP_TO_CHILD: string = "pitaya-page-transition--top-to-child";
const _TRANSITION_CHILD_TO_TOP: string = "pitaya-page-transition--child-to-top";
const _TRANSITIONS                     = [
	_TRANSITION_TOP_LEVEL,
	_TRANSITION_TOP_TO_CHILD,
	_TRANSITION_CHILD_TO_TOP
];

// const _VIEW_ENTERING_CLASS: string = "au-enter-active";
// const _VIEW_LEAVING_CLASS: string  = "au-leave-active";
const _VIEW_CONTROLS_CLASS: string = "pitaya-app-view--controls-blocked";

const _META_THEME_COLOR_SELECTOR: string = "meta[name=theme-color]";
// const _PRIMARY_COLOR_CSS_VARIABLE_NAME: string = "--mdc-theme-primary";

// const _scrollRestorationList: number[] = [];

let _currentAppInstance: PitayaApp        = null;
let _backButtonOverrideCallback: Function = null;
let _enableScroll: boolean                = true;

export const overrideBrowserBackButton = function( callback: Function )
{
	if( _backButtonOverrideCallback )
	{
		window.removeEventListener( "popstate", _backButtonOverrideCallback as EventListenerOrEventListenerObject );

		_backButtonOverrideCallback = null;
	}

	if( !callback )
	{
		window.history.back();

		return;
	}

	const handler = function()
	{
		window.history.pushState( {}, "" );

		callback();
	};

	_backButtonOverrideCallback = handler;

	window.addEventListener( "popstate", handler );

	window.history.pushState( {}, "" );
};

export const enableScrolling = function( enable: boolean = true )
{
	const enableScroll = typeof enable === "boolean" ? enable : !_enableScroll;
	const { body }     = document;

	_enableScroll = enableScroll;

	if( enableScroll )
	{
		body.style.removeProperty( "overflow" );
		body.style.removeProperty( "height" );

		return;
	}

	body.style.setProperty( "overflow", "hidden" );
	body.style.setProperty( "height", "100%" );
};

export const setStatusBarColor = function( color: string )
{
	_currentAppInstance.setStatusBarColor.call( _currentAppInstance, color );
};

export const navigate = function( routeName: string, options?: any )
{
	return _currentAppInstance.navigate.call( _currentAppInstance, routeName, options );
};

export const enableControls = function( enable: boolean = true )
{
	_currentAppInstance.enableControls = enable;
};

@inject( EventAggregator )
export abstract class PitayaApp
{
	static get instance()
	{
		return _currentAppInstance;
	}

	protected _router: Router;
	protected _routerConfig: RouterConfiguration;
	protected _settings                             = {
		title         : "" as string,
		configuration : {} as string | IAppConfig,
		routes        : {} as string | IRoutesConfigRoute[],
	};
	private _themeColorMetaElement: HTMLMetaElement = document.querySelector( _META_THEME_COLOR_SELECTOR );
	private _viewTransition: string                 = "";

	get viewTransition(): string
	{
		return this._viewTransition;
	}

	set viewTransition( value: string )
	{
		const { rootElement } = this._appView;

		if( !rootElement )
		{
			return;
		}

		this._viewTransition = value;

		const { classList } = rootElement;

		if( classList.contains( value ) )
		{
			return;
		}

		classList.remove( ..._TRANSITIONS );
		classList.add( value );
	}

	private _appView: PitayaAppViewCustomElement;

	get appView(): PitayaAppViewCustomElement
	{
		return this._appView;
	}

	private _enableControls: boolean = false;

	get enableControls(): boolean
	{
		return this._enableControls;
	}

	set enableControls( value: boolean )
	{
		const method = value ? "remove" : "add";

		this._enableControls = value;

		this._appView.rootElement.classList[ method ]( _VIEW_CONTROLS_CLASS );
	}

	get enableScroll(): boolean
	{
		return _enableScroll;
	}

	get currentView(): PitayaView
	{
		return PitayaView.instance || this._router.currentInstruction.viewPortInstructions.default.component.viewModel;
	}

	get isOnline()
	{
		return this._appView.isOnline;
	}

	get theme()
	{
		return this._themeColorMetaElement.getAttribute( "content" );
	}

	protected constructor( private _eventAggregator: EventAggregator )
	{
		_currentAppInstance = this;
	}

	/**
	 * @aurelia hook
	 * Configures global routes and router settings
	 */
	public async configureRouter( config: RouterConfiguration, router: Router ): Promise<void>
	{
		const routs         = await this._getRoutes();
		const [ mainRoute ] = routs;

		/* This adjustment is necessary to prevent aurelia from interfering with our scroll restoration mechanism */
		// ( router.history as any ).history.scrollRestoration = "manual";

		config.title = this._settings.title;
		config.map( routs );
		config.mapUnknownRoutes( mainRoute.moduleId );

		this._router       = router;
		this._routerConfig = config;

		this._setRouteChangeHandler( config );
		// config.addPostRenderStep( _postRenderStep );

		this[ _ROUTER_CONFIGURED_CALLBACK_NAME ] && this[ _ROUTER_CONFIGURED_CALLBACK_NAME ]();
	}

	/**
	 * @aurelia hook
	 */
	public async activate(): Promise<void>
	{
		await this._initSettings();
	}

	/**
	 * @aurelia hook
	 */
	public created( parentContext, context )
	{
		this._appView = context.controllers.find( controller => controller.behavior.elementName === _APP_VIEW_HTML_ELEMENT_NAME ).viewModel;
	}

	/**
	 * @aurelia hook
	 */
	public bind()
	{
		this._initNetworkChangeHandler();
	}

	/**
	 * @aurelia hook
	 */
	public attached()
	{

	}

	/* TODO: Neu machen! */
	public async navigate( routeName: string, options?: any )
	{
		if( !this._router.hasRoute( routeName ) )
		{
			if( !_isExternalLink( routeName ) )
			{
				throw new Error( `Navigation: Route '${routeName}' not found.` );
			}

			const win = window.open( routeName, "_blank" );

			win.focus();

			return;
		}

		const { route, nav } = this._router.routes.find( ( { name } ) => name === routeName );

		/* There is a bug when changing routes the aurelia way, so I just do it manually */
		location.href = `#/${route}`;
	}

	public navigateBack()
	{
		return this._router.navigateBack();
	}

	public setStatusBarColor( color: string )
	{
		this._themeColorMetaElement.setAttribute( "content", color );
	}

	private _setRouteChangeHandler( config: RouterConfiguration )
	{
		this._eventAggregator.subscribe( "router:navigation:processing", this._handleRouteProcessing.bind( this ) );
		this._eventAggregator.subscribe( "router:navigation:success", this._handleRouteSuccess.bind( this ) );
	}

	private _handleRouteProcessing( { instruction } )
	{
		if( !instruction.previousInstruction )
		{
			return;
		}

		const oldOrder = 1 * ( instruction.previousInstruction.config.navModel.order || 0 );
		const newOrder = 1 * ( instruction.config.navModel.order || 0 );

		if( oldOrder > newOrder )
		{
			this.viewTransition = _TRANSITION_CHILD_TO_TOP;
		}
		else if( oldOrder < newOrder )
		{
			this.viewTransition = _TRANSITION_TOP_TO_CHILD;
		}
		else
		{
			this.viewTransition = _TRANSITION_TOP_LEVEL;
		}
	}

	private _handleRouteSuccess( event: Event )
	{
		this._appView.currentViewName = this._router.currentInstruction.config.name;
	}

	private async _initSettings()
	{
		if( !this._settings.configuration )
		{
			return;
		}

		let settings;

		if( typeof this._settings.configuration === "string" )
		{
			const file = await fetch( this._settings.configuration );
			settings   = await file.json();
		}

		PitayaAppConfiguration.set( settings || this._settings.configuration );
	}

	private async _getRoutes(): Promise<IRoutesConfigRoute[]>
	{
		if( typeof this._settings.routes === "string" )
		{
			const file      = await fetch( this._settings.routes );
			const { routs } = await file.json();

			return routs;
		}

		return this._settings.routes;
	}

	private _initNetworkChangeHandler()
	{
		this._appView.isOnline = navigator.onLine;

		window.addEventListener( "online", this._onOnline.bind( this ) );
		window.addEventListener( "offline", this._onOffline.bind( this ) );
	}

	private async _onOnline()
	{
		this._appView.isOnline = true;

		const { currentView } = this;

		if( _ONLINE_CALLBACK_NAME in currentView )
		{
			return currentView[ _ONLINE_CALLBACK_NAME ]();
		}

		( _ONLINE_CALLBACK_NAME in this ) && this[ _ONLINE_CALLBACK_NAME ]();

		await delay( 100 );

		PitayaView.instance.setTheme();
	}

	private async _onOffline()
	{
		this._appView.isOnline = false;

		const { currentView } = this;

		if( _OFFLINE_CALLBACK_NAME in currentView )
		{
			return currentView[ _OFFLINE_CALLBACK_NAME ]();
		}

		( _OFFLINE_CALLBACK_NAME in this ) && this[ _OFFLINE_CALLBACK_NAME ]();

		await delay( 100 );

		PitayaView.instance.setTheme();
	}
}

/**
 * @see Jon´s answer -> https://stackoverflow.com/questions/6238351/fastest-way-to-detect-external-urls
 * @param link {string}
 */
const _isExternalLink = function( link: string ): boolean
{
	const locationHref = location.href.replace( "http://", "" )
								 .replace( "https://", "" )
								 .split( "/" )[ 0 ];

	const refinedLink = link.replace( "http://", "" ).replace( "https://", "" )
							.split( "/" )[ 0 ];

	return locationHref !== refinedLink;
};
